/*
 * Copyright (C) 2002 Don Dugger <n0ano@n0ano.com>
 */

/*
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Library General Public License for more details.
 * A copy of the GNU General Public License should have been
 * included in the file `gpl.txt' as part of this source distribution.
 * If you cannot find this file you my obtain a copy of the GNU
 * General Public License by writing to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */
#
#define DO_NOT_ALLOW_ACCESS_TO_INTERNALS_OF_STRUCTS
#include <BuildDefines.h>
#ifdef DEBUG_BUILD
#define ERROR_CHECK_LEVEL ERROR_CHECK_FULL
#endif
#include <PalmOS.h>
#include "resources.h"
#include "bj.h"
#include "utils.h"
#include "msgs.h"
#include "scroll.h"

struct bjstate bjstate;

char buf[64];

#define X(p)	(p.topLeft.x)
#define Y(p)	(p.topLeft.y)
#define W(p)	(p.extent.x)
#define H(p)	(p.extent.y)

void
ShowCard(FormPtr fp, int id, int v)
{
	int idx;
	RectangleType b;

	idx = FrmGetObjectIndex(fp, id);
	FrmGetObjectBounds(fp, idx, &b);
	WinEraseRectangle(&b, 0);
	if (v) {
		X(b) += 2;
		W(b) -= 3;
		Y(b) += 1;
		H(b) -= 2;
		WinDrawRectangleFrame(1, &b);
		if (v & 0x80) {
			WinDrawLine(X(b), Y(b), X(b) + W(b), Y(b) + H(b));
			WinDrawLine(X(b), Y(b) + H(b), X(b) + W(b), Y(b));
		} else {
			WinDrawChar(DISP_VAL(v), X(b) + 4, Y(b) + 2);
			WinDrawChar(DISP_SUIT(v), X(b) + 2, Y(b) + 12);
		}
	}
	return;
}

int count_cards(unsigned char *cds, int high)
{
	int i, v, c, aces;

	c = 0;
	aces = 0;
	for (i = 0; i < MAXCARDS; i++) {
		switch (v = VAL(*cds++)) {

		case 2:
		case 3:
		case 4:
		case 5:
		case 6:
		case 7:
		case 8:
		case 9:
		case 10:
			c += v;
			break;

		case 11:
		case 12:
		case 13:
			c += 10;
			break;

		case 14:
			c += 1;
			if (high)
				aces += 10;
			break;

		}
	}
	while (aces) {
		if ((c + aces) <= 21)
			break;
		else
			aces -= 10;
	}
	return (c + aces);
}

static char *
ShCount(unsigned char *cds)
{

	buf[0] = '\0';
	if (cds[0] & 0x80)
		return (buf);
	return (itoa(Count(cds), buf));
}

void
ShowCards(FormPtr fp, unsigned char *pcd)
{
	int i;

	for (i = 0; i < MAXCARDS; i++) {
		ShowCard(fp, DealerCard1 + i, DealerCards[i]);
		ShowCard(fp, PlayerCard1 + i, pcd[i]);
	}
	setfield(fp, FieldDCount, ShCount(DealerCards), 1);
	setfield(fp, FieldPCount, ShCount(pcd), 1);
	return;
}

void
shmsg(int mid)
{
	FormPtr fp;

	fp = FrmGetActiveForm();
	Msgid = mid;
	setfield(fp, FieldMsg, Msgs[mid], 1);
	return;
}

static int ask(int id)
{
	FormPtr form, fp;
	int r;

	form = FrmGetActiveForm();
	fp = FrmInitForm(id);
	FrmSetActiveForm(fp);
	r = FrmDoDialog(fp);
	if (form)
		FrmSetActiveForm(form);
	FrmDeleteForm(fp);
	return(r);
}

NewGame(int first)
{
	int i;

	if (first) {
		StartPot = 500;
		Bet = 10;
		BetMin = 10;
		BetMax = 20;
		DefaultRules = 0;	/* Vegas */
		DealerSoft17 = 1;
		DealerResplit = 1;
		DealerSplitace = 1;
		DealerDblSplit = 0;
		DblRule = 0;		/* Any */
		DealerDblAny = 1;
		DealerDblTen = 0;
		DealDelay = 0;
		DefDecks = 0;
		Ndecks = 1;
		DefDepth = 0;
		Depth = 80;
		Surrender = 0;
		Nanny = 0;
	}
	Hands = 0;
	Hwon = 0;
	Hlost = 0;
	Money = 0;
	Mwon = 0;
	Decisions = 0;
	Dwon = 0;
	Pot = StartPot;
	OldPot = Pot;
	Curbet = PlayerBets;
	*Curbet = 0;
	Winnings = 0;
	Deal = 0;
	Msgid = 0;
	NextCard = &Deck[MAXDECK * 52];
	Decklim = 0;
	PlayerCards = &PlayerHands[0][0];
	for (i = 0; i < MAXCARDS; i++) {
		DealerCards[i] = 0;
		PlayerCards[i] = 0;
	}
	BjState = BJ_DONE;
	return;
}

void NewHands()
{
	DmOpenRef db;
	MemHandle h;
	struct bjstate *bp;
	UInt16 ri;

	ri = 0;
	if ((db = DmOpenDatabaseByTypeCreator('DATA', 'xyz1', dmModeReadWrite)) == NULL) {
		NewGame(1);
		if (DmGetLastErr() == dmErrCantFind)
			DmCreateDatabase(0, "PBJ-State", 'xyz1', 'DATA', 0);
		db = DmOpenDatabaseByTypeCreator('DATA', 'xyz1', dmModeReadWrite);
		h = DmNewRecord(db, &ri, sizeof(struct bjstate));
		bp = (struct bjstate *)MemHandleLock(h);
		DmWrite(bp, 0, &bjstate, sizeof(struct bjstate));
	} else {
		h = DmGetRecord(db, 0);
		bp = (struct bjstate *)MemHandleLock(h);
		bjstate = *bp;
	}
	MemHandleUnlock(h);
	DmReleaseRecord(db, ri, 1);
	DmCloseDatabase(db);
	return;
}

#define MAXRAND	((unsigned long)0xffffffff)

int getrand(int n)
{
	unsigned long genrand();
	int r;

	do {
		r = (int)((float)n * (float)genrand()/((float)MAXRAND + 1.0));
	} while (r >= n);
	return(r);
}

static int shuffle()
{
	int i, j, r;
	unsigned char *cp, c;

	if (ask(QueryShuffle) == 0)
		return(0);
	cp = Deck;
	Ncards = Ndecks * 52;
	/*
	 *  Following is the same as (Ncards * Depth) / 100
	 */
	Decklim = (Ndecks * Depth / 2);
	i = 2;
	j = 0;
	while (cp < &Deck[Ncards]) {
		*cp++ = CARD(i, j);
		if (++i > 14) {
			i = 2;
			j = (j + 1) & 0x3;
		}
	}
	for (i = 0; i < NSHUFFLE; i++)
		for (j = 0; j < Ncards; j++) {
			r = getrand(Ncards);
			c = Deck[j];
			Deck[j] = Deck[r];
			Deck[r] = c;
		}
	NextCard = Deck;
	return(1);
}

static int options(unsigned char *cds, int ncards)
{
	int extra;

	if (ncards > 2)
		return(0);
	extra = 0;
	if (First && Surrender)
		extra |= 4;
	if (ORD(cds[0]) == ORD(cds[1])) {
		if (First || DealerResplit)
			extra |= 1;
	}
	if (DealerDblSplit == 0 && First == 0)
		return(extra);
	switch (CountMin(cds)) {

	case 11:  case 10:
		extra |= 2;
		break;

	case 9:
		if (DealerDblTen == 0)
			extra |= 2;
		break;

	default:
		if (DealerDblAny)
			extra |= 2;
		break;

	}
	return(extra);
}

static int dialog(unsigned char *cds, int ncards)
{
	FormPtr form, fp;
	int extra, r;

	extra = options(cds, ncards);
	form = FrmGetActiveForm();
	fp = FrmInitForm(QueryDialog);
	if ((extra & 1) == 0)
		FrmHideObject(fp, FrmGetObjectIndex(fp, QuerySplitButton));
	if ((extra & 2) == 0)
		FrmHideObject(fp, FrmGetObjectIndex(fp, QueryDoubleButton));
	if ((extra & 4) == 0)
		FrmHideObject(fp, FrmGetObjectIndex(fp, QuerySurrButton));
	FrmSetActiveForm(fp);
	r = FrmDoDialog(fp);
	if (form)
		FrmSetActiveForm(form);
	FrmDeleteForm(fp);
	return(r);
}

static int shhint(int formid)
{
	FormPtr fp, form;
	int r;

	form = FrmGetActiveForm();
	fp = FrmInitForm(formid);
	setfield(fp, FieldHint, hintstr(PlayerCards, ORD(DealerCards[1]), PlayerNext - PlayerCards), 0);
	FrmSetActiveForm(fp);
	r = FrmDoDialog(fp);
	if (form)
		FrmSetActiveForm(form);
	FrmDeleteForm(fp);
	return(r);
}

static int observe(unsigned char *cds, int ncards)
{
	int i, h, r;

	if ((i = dialog(cds, ncards)) == 0 || i == QueryHintButton)
		return(i);
	h = hint(cds, ORD(DealerCards[1]), ncards);
	Decisions++;
	if (hintid(h) == i)
		Dwon++;
	else if (Nanny) {
		if (shhint(QuerySure) == 0)
			return(0);
		i = dialog(cds, ncards);
	}
	return(i);
}

static void result(int win, FormPtr fp)
{

	if (win == 0)
		setfield(fp, FieldResult, "", 1);
	else if (win > 0)
		setfield(fp, FieldResult, "Won:", 1);
	else {
		win = -win;
		setfield(fp, FieldResult, "Lost:", 1);
	}
	setfield(fp, FieldWinnings, itoa(win, buf), 1);
	return;
}

void refresh()
{
	FormPtr fp;

	fp = FrmGetActiveForm();
	ShowCards(fp, PlayerCards);
	shmsg(Msgid);
	setfield(fp, FieldPot, itoa(Pot, buf), 1);
	setfield(fp, FieldBet, itoa(*Curbet, buf), 1);
	result(Winnings, fp);
	FrmDrawForm(fp);
	return;
}

static int dealcard()
{

	First = 0;
	return(*NextCard++);
}

static int deal()
{
	int i;

	PlayerCards = &PlayerHands[0][0];
	for (i = 0; i < MAXCARDS; i++) {
		PlayerCards[i] = 0;
		DealerCards[i] = 0;
	}
	if (NextCard >= &Deck[Decklim]) {
		shmsg(0);
		refresh();
		if (shuffle() == 0)
			return(0);
	}
	PlayerCards[0] = dealcard();
	DealerCards[0] = dealcard() | 0x80;
	PlayerCards[1] = dealcard();
	DealerCards[1] = dealcard();
	shmsg(0);
	refresh();
	return(1);
}

static int do_dealer()
{
	unsigned char *cp;
	int i;

	cp = DealerCards + 2;
	while ((i = Count(DealerCards)) <= 17) {
		if (i == 17 && CountMin(DealerCards) == 17)
			break;
		*cp++ = dealcard();
		refresh();
		delay(100);
	}
	return(i);
}

static int setbet(int strat, int bet, int min, int max)
{

	switch (strat) {

	case 0:		/* None */
		break;

	case 1:		/* Martingale */
		if (Pot < OldPot)
			return(bet * 2);
		break;

	case 2:		/* Grind 1,2 */
		if (Pot > OldPot)
			return(max);
		break;

	case 3:		/* Grind 1,2,3,5 */
		if (Pot > OldPot) {
			if (bet <= min)
				return(min * 2);
			else if (bet <= (min * 2))
				return(min * 3);
			else if (bet <= (min * 3))
				return(min * 5);
		}

	}
	return(Pot == OldPot ? bet : min);
}

static int resolve(int mon, int won, int msg)
{

	Pot += mon;
	if (won > 0)
		Hwon++;
	else if (won < 0)
		Hlost++;
	DealerCards[0] &= ~0x80;
	Winnings = Pot - OldPot;
	if (Winnings > 0)
		Mwon += Winnings;
	BjState = BJ_DONE;
	Msgid = msg;
	refresh();
	return(0);
}

int play(FormPtr form)
{
	FormPtr fp;
	int i, pc, dc, r;
	unsigned char cd, *cds;

	switch (BjState) {

	case BJ_START:
		if (deal() == 0)
			return(Msgid);
		Bet = setbet(BetStrat, Bet, BetMin, BetMax);
		Money += Bet;
		Curbet = PlayerBets;
		*Curbet = Bet;
		OldPot = Pot;
		Winnings = 0;
		Hands++;
		Pot -= *Curbet;
		/*
		 *  Insurance
		 */
		if (VAL(DealerCards[1]) == 14)
			BjState = BJ_INSURANCE;
		else
			BjState = BJ_CKFOR21;
		return(1);

	case BJ_INSURANCE:
		if ((r = ask(Surrender == 1 ? QuerySurr : QueryInsur)) == QueryOK) {
			Pot -= *Curbet >> 1;
			if (Count(DealerCards) == 21) {
				Pot += (*Curbet >> 1) * 2;
				DealerCards[0] &= ~0x80;
				shmsg(MSG_DEALERBJ);
			}
			BjState = BJ_SHINSUR;
			return(1);
		} else if (r == QuerySurrButton)
			return(resolve(*Curbet/2, -1, MSG_SURR));
		else if (r == 0)
			return(0);
		BjState = BJ_CKFOR21;
		return(1);

	case BJ_SHINSUR:
		if (ask(Count(DealerCards) == 21 ? QueryInsurWins : QueryInsurLose) == 0)
			return(0);
		BjState = BJ_CKFOR21;
		/* Fall thru */

	case BJ_CKFOR21:
		/*
		 *  Does Dealer have 21
		 */
		if (Count(DealerCards) == 21) {
			if (Count(PlayerCards) != 21)
				resolve(0, -1, MSG_DEALERBJ);
			else
				resolve(*Curbet, 0, MSG_PUSH21);
			return(0);
		}
		if (Count(PlayerCards) == 21)
			return(resolve(*Curbet + *Curbet + (*Curbet >> 1), 1, MSG_BJ));
		BjState = BJ_DOPLAYER;
		First = 1;
		PlayerCardsMax = PlayerCards;
		PlayerNext = &PlayerCards[2];
		/* Fall thru */

	case BJ_DOPLAYER:
		switch (observe(PlayerCards, PlayerNext - PlayerCards)) {

		case QueryHitButton:
			*PlayerNext++ = dealcard();
			if (Count(PlayerCards) > 21)
				if (PlayerCards != PlayerCardsMax)
					BjState = BJ_NEXTSPLIT;
				else
					BjState = BJ_DODEALER;
			break;

		case QueryStandButton:
			if (PlayerCards != PlayerCardsMax)
				BjState = BJ_NEXTSPLIT;
			else
				BjState = BJ_DODEALER;
			break;

		case QueryDoubleButton:
			Pot -= Bet;
			*Curbet += Bet;
			*PlayerNext++ = dealcard();
			if (PlayerCards != PlayerCardsMax)
				BjState = BJ_NEXTSPLIT;
			else
				BjState = BJ_DODEALER;
			break;

		case QuerySplitButton:
			PlayerCardsMax += MAXCARDS;
			Pot -= Bet;
			Hands++;
			Money += Bet;
			PlayerCardsMax[0] = PlayerCards[1];
			for (i = 2; i < MAXCARDS; i++)
				PlayerCardsMax[i] = 0;
			if (VAL(PlayerCards[0]) == 14 && DealerSplitace)
				BjState = BJ_SPLITACE1;
			else
				BjState = BJ_FIRSTSPLIT;
			break;

		case QueryHintButton:
			BjHintState = BjState;
			BjState = BJ_HINT;
			break;

		case QuerySurrButton:
			return(resolve(*Curbet >> 1, -1, MSG_SURR));

		case 0:
			return(0);

		}
		return(1);

	case BJ_HINT:
		shhint(QueryHint);
		BjState = BjHintState;
		return(1);

	case BJ_FIRSTSPLIT:
		if (ask(QueryFirstSplit) == 0)
			return(0);
		PlayerCards[1] = dealcard();
		PlayerNext = &PlayerCards[2];
		BjState = BJ_DOPLAYER;
		return(1);

	case BJ_NEXTSPLIT:
		if (ask(Count(PlayerCards) > 21 ? QueryNextBusted : QueryNextHand) == 0)
			return(0);
		if (Count(PlayerCards) > 21) {
			cds = PlayerCards;
			while (cds < PlayerCardsMax) {
				cds[0] = cds[MAXCARDS];
				cds += MAXCARDS;
			}
			for (i = 2; i < MAXCARDS; i++)
				PlayerCards[i] = 0;
			PlayerCardsMax -= MAXCARDS;
		} else {
			PlayerCards += MAXCARDS;
			Curbet++;
		}
		*Curbet = Bet;
		PlayerCards[1] = dealcard();
		PlayerNext = &PlayerCards[2];
		BjState = BJ_DOPLAYER;
		break;

	case BJ_SPLITACE1:
		if (ask(QuerySplitAce) == 0)
			return(0);
		PlayerCards[1] = dealcard();
		BjState = BJ_SPLITACE2;
		break;

	case BJ_SPLITACE2:
		if (ask(QuerySplitAce) == 0)
			return(0);
		PlayerCards += MAXCARDS;
		PlayerCards[1] = dealcard();
		*++Curbet = Bet;
		Pot -= Bet;
		BjState = BJ_DODEALER;
		/* Fall thru */

	case BJ_DODEALER:
		DealerCards[0] &= ~0x80;
		refresh();
		delay(100);
		if (PlayerCards == &PlayerHands[0][0] && Count(PlayerCards) > 21)
			return(resolve(0, -1, MSG_BUST));
		dc = do_dealer();
		PlayerCardsMax = PlayerCards;
		PlayerCards = &PlayerHands[0][0];
		refresh();
		BjState = BJ_SHOWDOWN;
		/* Fall thru */

	case BJ_SHOWDOWN:
		pc = Count(PlayerCards);
		dc = Count(DealerCards);
		if (dc > 21) {
			Winnings = *Curbet + *Curbet;
			r = 1;
			Msgid = MSG_DBUST;
		} else if (pc > dc) {
			Winnings = *Curbet + *Curbet;
			r = 1;
			Msgid = MSG_WIN;
		} else if (pc == dc) {
			Winnings += *Curbet;
			r = 0;
			Msgid = MSG_PUSH;
		} else {
			r = -1;
			Msgid = MSG_LOSE;
		}
		if (PlayerCards == PlayerCardsMax)
			return(resolve(Winnings, r, Msgid));
		refresh();
		if (ask(QueryCheckHand) == 0)
			return(0);
		resolve(Winnings, r, Msgid);
		BjState = BJ_SHOWDOWN;
		PlayerCards += MAXCARDS;
		return(1);

	}
	return(BjState);
}
