/*
 * Copyright (C) 2002 Don Dugger <n0ano@n0ano.com>
 */

/*
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Library General Public License for more details.
 * A copy of the GNU General Public License should have been
 * included in the file `gpl.txt' as part of this source distribution.
 * If you cannot find this file you my obtain a copy of the GNU
 * General Public License by writing to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */
#ifndef BJ_H
#define BJ_H

/*
 *  Game states
 */

#define BJ_DONE		0
#define BJ_START	1
#define BJ_INSURANCE	2
#define BJ_SHINSUR	3
#define BJ_CKFOR21	4
#define BJ_DOPLAYER	5
#define BJ_HINT		6
#define BJ_FIRSTSPLIT	7
#define BJ_NEXTSPLIT	8
#define BJ_SPLITACE1	9
#define BJ_SPLITACE2	10
#define BJ_DODEALER	11
#define BJ_SHOWDOWN	12

#define NSHUFFLE	4
#define MAXCARDS	10
#define MAXDECK		8

#define CARD(v,s)	((s) | ((v) << 2))
#define SUIT(c)		((c) & 0x3)
#define VAL(c)		(((c) >> 2) & 0xf)
#define ORD(c)		(VAL(c) == 14 ? 11 : ((VAL(c) > 9 && VAL(c) < 14) ? 10 : VAL(c)))

#define DISP_VAL(c)	"23456789TJQKA"[VAL(c) - 2]
#define DISP_SUIT(c)	(0x8D + SUIT(c))

#define Count(cds)	count_cards(cds, 1)
#define CountMin(cds)	count_cards(cds, 0)

struct popup {
	char *name;
};

struct bjstate {
	/*
	 * State
	 */
	int state;
	int hintstate;
	int nanny;
	int startpot;
	int pot;
	int oldpot;
	int bet;
	int betmin;
	int betmax;
	int *curbet;
	int winnings;
	int betstrat;
	int deal;
	int msgid;
	int first;
	int ncards;
	int defdecks;
	int ndecks;
	int defdepth;
	int depth;
	int decklim;

	/*
	 * Statistics
	 */
	int hands;
	int hwon;
	int hlost;
	int money;
	int mwon;
	int decisions;
	int dwon;

	/*
	 * Rules
	 */
	char defaultrules;
	char dealdelay;
	char dealersoft17;
	char dealerresplit;
	char dealersplitace;
	char dealerdblsplit;
	char dblrule;
	char dealerdblany;
	char dealerdblten;
	char surrender;

	/*
	 * Cards
	 */
	unsigned char deck[MAXDECK * 52];
	unsigned char *nextcard;
	unsigned char dealercards[MAXCARDS];
	unsigned char playerhands[4][MAXCARDS];
	unsigned int  playerbets[4];
	unsigned char *playercards;
	unsigned char *playernext;
	unsigned char *playercardsmax;
};
#define BjState		bjstate.state
#define BjHintState	bjstate.hintstate
#define Nanny		bjstate.nanny
#define StartPot	bjstate.startpot
#define Pot		bjstate.pot
#define OldPot		bjstate.oldpot
#define Bet		bjstate.bet
#define BetMin		bjstate.betmin
#define BetMax		bjstate.betmax
#define Curbet		bjstate.curbet
#define Winnings	bjstate.winnings
#define BetStrat	bjstate.betstrat
#define Deal		bjstate.deal
#define Msgid		bjstate.msgid
#define First		bjstate.first
#define Ncards		bjstate.ncards
#define DefDecks	bjstate.defdecks
#define Ndecks		bjstate.ndecks
#define DefDepth	bjstate.defdepth
#define Depth		bjstate.depth
#define Decklim		bjstate.decklim
#define Deck		bjstate.deck
#define Hands		bjstate.hands
#define Hwon		bjstate.hwon
#define Hlost		bjstate.hlost
#define Money		bjstate.money
#define Mwon		bjstate.mwon
#define Decisions	bjstate.decisions
#define Dwon		bjstate.dwon
#define DefaultRules	bjstate.defaultrules
#define DealDelay	bjstate.dealdelay
#define DealerSoft17	bjstate.dealersoft17
#define DealerResplit	bjstate.dealerresplit
#define DealerSplitace	bjstate.dealersplitace
#define DealerDblSplit	bjstate.dealerdblsplit
#define DblRule		bjstate.dblrule
#define DealerDblAny	bjstate.dealerdblany
#define DealerDblTen	bjstate.dealerdblten
#define Surrender	bjstate.surrender
#define NextCard	bjstate.nextcard
#define DealerCards	bjstate.dealercards
#define PlayerHands	bjstate.playerhands
#define PlayerBets	bjstate.playerbets
#define PlayerCards	bjstate.playercards
#define PlayerNext	bjstate.playernext
#define PlayerCardsMax	bjstate.playercardsmax

extern struct bjstate bjstate;

extern char buf[64];

void NewHands();
void refresh();
int count_cards(unsigned char *cds, int high);
int play(FormPtr form);
char *hintstr(unsigned char *pcd, int dcd, int ncards);
int hint(unsigned char *pcd, int dcd, int ncards);
int hintid(int h);
void menupref();
void menurules();

#endif	// BJ_H
