/*
 * Copyright (C) 2002 Don Dugger <n0ano@n0ano.com>
 */

/*
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Library General Public License for more details.
 * A copy of the GNU General Public License should have been
 * included in the file `gpl.txt' as part of this source distribution.
 * If you cannot find this file you my obtain a copy of the GNU
 * General Public License by writing to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */
#
#define DO_NOT_ALLOW_ACCESS_TO_INTERNALS_OF_STRUCTS
#include <BuildDefines.h>
#ifdef DEBUG_BUILD
#define ERROR_CHECK_LEVEL ERROR_CHECK_FULL
#endif
#include <PalmOS.h>
#include "bj.h"
#include "resources.h"

#define	S	0
#define H	1
#define D	2
#define P	3
#define G	4	/* Surrender - give up */

struct action {
	char *name;
	int val;
} action[] = {
/* S */	"    Stand",	QueryStandButton,
/* H */	"      Hit",	QueryHitButton,
/* D */	"   Double",	QueryDoubleButton,
/* P */	"     Split",	QuerySplitButton,
/* G */ "Surrender",	QuerySurrButton
};

#define SS	((S << 2) | S)
#define HH	((H << 2) | H)
#define DD	((D << 2) | D)
#define PS	((P << 2) | S)
#define PH	((P << 2) | H)
#define DH	((D << 2) | H)
#define DS	((D << 2) | S)
#define PD	((P << 2) | D)

static char sphints[10][10] = {
/*
Player
cards   Dealer up card
	 2   3   4   5   6   7   8   9   X   A */
/*22*/	HH, PH, PH, PH, PH, PH, HH, HH, HH, HH,
/*33*/  HH, HH, PH, PH, PH, PH, HH, HH, HH, HH,
/*44*/	HH, HH, HH, DH, DH, HH, HH, HH, HH, HH,
/*55*/	DH, DH, DH, DH, DH, DH, DH, DH, HH, HH,
/*66*/	PH, PS, PS, PS, PS, HH, HH, HH, HH, HH,
/*77*/	PS, PS, PS, PS, PS, PH, HH, HH, SS, HH,
/*88*/	PS, PS, PS, PS, PS, PH, PH, PH, PH, PH,
/*99*/	PS, PS, PS, PS, PS, SS, PS, PS, SS, SS,
/*XX*/	SS, SS, SS, SS, SS, SS, SS, SS, SS, SS,
/*AA*/	PH, PH, PH, PD, PD, PH, PH, PH, PH, PH
};

static char hints[17][10] = {
/*
Player
cards   Dealer up card
	 2   3   4   5   6   7   8   9   X   A */
/* 8*/	HH, HH, HH, DH, DH, HH, HH, HH, HH, HH,
/* 9*/	DH, DH, DH, DH, DH, HH, HH, HH, HH, HH,
/*10*/	DH, DH, DH, DH, DH, DH, DH, DH, HH, HH,
/*11*/	DH, DH, DH, DH, DH, DH, DH, DH, DH, DH,
/*12*/	HH, HH, SS, SS, SS, HH, HH, HH, HH, HH,
/*13*/	SS, SS, SS, SS, SS, HH, HH, HH, HH, HH,
/*14*/	SS, SS, SS, SS, SS, HH, HH, HH, HH, HH,
/*15*/	SS, SS, SS, SS, SS, HH, HH, HH, HH, HH,
/*16*/	SS, SS, SS, SS, SS, HH, HH, HH, HH, HH,
/*A2*/	HH, HH, DH, DH, DH, HH, HH, HH, HH, HH,
/*A3*/  HH, HH, DH, DH, DH, HH, HH, HH, HH, HH,
/*A4*/	HH, HH, DH, DH, DH, HH, HH, HH, HH, HH,
/*A5*/	HH, HH, DH, DH, DH, HH, HH, HH, HH, HH,
/*A6*/	DH, DH, DH, DH, DH, HH, HH, HH, HH, HH,
/*A7*/	SS, DS, DS, DS, DS, SS, SS, HH, HH, HH,
/*A8*/	SS, SS, SS, SS, DS, SS, SS, SS, SS, SS,
/*A9*/	SS, SS, SS, SS, SS, SS, SS, SS, SS, SS,
/*AT*/	SS, SS, SS, SS, SS, SS, SS, SS, SS, SS
};

static int cansplit()
{

	if (First || DealerResplit)
		return(1);
	return(0);
}

static int candbl(unsigned char *pcd, int nc)
{

	if (First == 0 && DealerDblSplit == 0)
		return(0);
	if (nc > 2)
		return(0);
	switch (CountMin(pcd)) {

	case 11:	case 10:
		return(1);

	case 9:
		if (DealerDblTen == 0)
			return(1);
		break;

	default:
		if (DealerDblAny)
			return(1);
		break;

	}
	return(0);
}

int split(unsigned char *pcd, int dcd)
{
	int h;

	if (VAL(pcd[0]) == 14)
		return(cansplit() ? P : H);
	h = sphints[ORD(*pcd) - 2][dcd];
	switch ((h >> 2) & 0x3) {

	case D:
		if (candbl(pcd, 2))
			return(D);
		break;

	case P:
		if (cansplit())
			return(P);
		break;

	}
	return(h & 0x3);
}

static int surrender(int nc, int pc, int mpc, int dcd)
{

	if (Surrender == 0 || First == 0 || nc > 2 || mpc != pc)
		return(0);
	if (pc == 15 && dcd == 10)
		return(1);
	if (pc == 16 && dcd >= 9)
		return(1);
	return(0);
}

int hint(unsigned char *pcd, int dcd, int nc)
{
	int h, pc, mpc;

	pc = Count(pcd);
	mpc = CountMin(pcd);
	if (surrender(nc, pc, mpc, dcd))
		return(G);
	dcd -= 2;
	if (ORD(pcd[0]) == ORD(pcd[1]) && nc == 2)
		return(split(pcd, dcd));
	if (mpc != pc)
		pc += 4;
	else if (pc > 16)
		return(S);
	else if (pc < 8)
		return(H);
	pc -= 8;
	h = hints[pc][dcd];
	if (((h >> 2) & 0x3) == D && candbl(pcd, nc))
		return(D);
	return(h & 0x3);
}

char *hintstr(unsigned char *pcd, int dcd, int nc)
{

	return(action[hint(pcd, dcd, nc)].name);
}

int hintid(int h)
{

	return(action[h].val);
}
