/*
 Copyright (c) 2000-2001, Neil Rhodes and Julie McKeehan
    neil@pobox.com
 All rights reserved.

From the book "Palm Programming, the Developer's Guide, 2nd edition"
by O'Reilly.

Permission granted to use this file however you see fit.
*/

/*
 Modified 2002 by Don Dugger <n0ano@n0ano.com>
 */

#define DO_NOT_ALLOW_ACCESS_TO_INTERNALS_OF_STRUCTS
#include <BuildDefines.h>
#ifdef DEBUG_BUILD
#define ERROR_CHECK_LEVEL ERROR_CHECK_FULL
#endif
#include <PalmOS.h>
#include "resources.h"
#include "bj.h"
#include "utils.h"
#include "constants.h"
#include "instructions.h"

static
    void
MainFormInit(FormPtr form)
{
#pragma unused(form)

	// warning-- don't do any drawing in this routine.
	// Also, don't call FrmSetFocus from here (it must be called *after*
	// FrmDrawForm)
}

static
    void
MainFormDeinit(FormPtr form)
{
#pragma unused(form)
}

static Boolean
MainFormHandleEvent(EventPtr event)
{
	DmOpenRef db;
	MemHandle h;
	Boolean handled = false;
	FormPtr form;
	FieldPtr field;
	struct bjstate *bp;
	char *cp, deck[2], dep[3];
	int idx, m, r;

	switch (event->eType) {
	case frmOpenEvent:
		form = FrmGetActiveForm();
		MainFormInit(form);
		FrmDrawForm(form);
		idx = FrmGetActiveFormID();
		if (idx == MainFormDeal) {
			if (BjState != BJ_DONE) {
				idx = FrmGetObjectIndex(form, MainDealButton);
				FrmHideObject(form, idx);
				refresh();
				while (play(form))
					refresh();
				FrmShowObject(form, idx);
			}
			refresh();
		} else {
			setfield(form, FieldInstr,
				idx == MenuInstr ? bjinstr : proginstr, 1);
			UpdateScrollbar(form, FieldInstr, ScrollInstr);
		}
		// here's where you'd add a call to FrmSetFocus
		handled = true;
		break;

	case ctlSelectEvent:
		switch (event->data.ctlSelect.controlID) {

		case MainDealButton:
			form = FrmGetActiveForm();
			idx = FrmGetObjectIndex(form, MainDealButton);
			FrmHideObject(form, idx);
			BjState = BJ_START;
			while (play(form))
				refresh();
			FrmShowObject(form, idx);
			Curbet = PlayerBets;
			*Curbet = 0;
			refresh();
			handled = true;
			break;

		case InstrButton:
			FrmGotoForm(MainFormDeal);
			handled = true;
			break;

		}

		break;

	case sclRepeatEvent:
		ScrollLines(FrmGetActiveForm(), FieldInstr, ScrollInstr,
				event->data.sclRepeat.newValue - event->data.sclRepeat.value, true);
		break;

	case menuEvent:
		switch (event->data.menu.itemID) {

		case MenuAbout:
			form = FrmInitForm(MenuAbout);
			FrmDoDialog(form);
			FrmDeleteForm(form);
			handled = true;
			break;

		case MenuInstr:
			FrmGotoForm(MenuInstr);
			handled = true;
			break;

		case MenuProgram:
			FrmGotoForm(MenuProgram);
			handled = true;
			break;

		case MenuNewGame:
			form = FrmInitForm(MenuNewGame);
			if (FrmDoDialog(form) == QueryOK) {
				NewGame(0);
				refresh();
			}
			FrmDeleteForm(form);
			handled = true;
			break;

		case MenuStats:
			menustats();
			handled = true;
			break;

		case MenuPref:
			menupref();
			handled = true;
			break;

		case MenuRules:
			menurules();
			handled = true;
			break;

		}
		break;

	case frmCloseEvent:
		db = DmOpenDatabaseByTypeCreator('DATA', 'xyz1', dmModeReadWrite);
		h = DmGetRecord(db, 0);
		bp = (struct bjstate *)MemHandleLock(h);
		DmWrite(bp, 0, &bjstate, sizeof(struct bjstate));
		MemHandleUnlock(h);
		DmReleaseRecord(db, 0, 1);
		DmCloseDatabase(db);
		MainFormDeinit(FrmGetActiveForm());
		handled = false;
		break;

	default:
		break;

	}
	return handled;
}

static Boolean
AppHandleEvent(EventPtr event)
{
	UInt16 formId;
	FormPtr form;

	if (event->eType == frmLoadEvent) {
		// Load the form resource.
		formId = event->data.frmLoad.formID;
		form = FrmInitForm(formId);
		ErrFatalDisplayIf(!form, "Can't initialize form");
		FrmSetActiveForm(form);

		// Set the event handler for the form.  The handler of the currently
		// active form is called by FrmHandleEvent each time is receives an
		// event.
		switch (formId) {

		case MainFormDeal:
			FrmSetEventHandler(form, MainFormHandleEvent);
			break;

		case MenuInstr:
			FrmSetEventHandler(form, MainFormHandleEvent);
			break;

		case MenuProgram:
			FrmSetEventHandler(form, MainFormHandleEvent);
			break;

		default:
			ErrFatalDisplay("Invalid Form Load Event");
			break;

		}
		return true;
	} else
		return false;
}

static
    void
AppEventLoop(void)
{
	Err error;
	EventType event;

	do {
		EvtGetEvent(&event, evtWaitForever);

		if (!SysHandleEvent(&event))
			if (!MenuHandleEvent(0, &event, &error))
				if (!AppHandleEvent(&event))
					FrmDispatchEvent(&event);

	} while (event.eType != appStopEvent);
}

static
    Err
AppStart(void)
{

	NewHands();
	sgenrand(TimGetTicks());
	return errNone;
}

static
    void
AppStop(void)
{
	FrmCloseAllForms();
}

UInt32
PilotMain(UInt16 launchCode, MemPtr launchParameters, UInt16 launchFlags)
{
#pragma unused(launchParameters)
	Err error;

	switch (launchCode) {
	case sysAppLaunchCmdNormalLaunch:
		error = RomVersionCompatible(kOurMinVersion, launchFlags);
		if (error)
			return error;
		error = AppStart();
		if (error)
			return error;

		FrmGotoForm(MainFormDeal);
		AppEventLoop();
		AppStop();
		break;

	default:
		break;
	}

	return errNone;
}
