/*
 * Copyright (C) 2002 Don Dugger <n0ano@n0ano.com>
 */

/*
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Library General Public License for more details.
 * A copy of the GNU General Public License should have been
 * included in the file `gpl.txt' as part of this source distribution.
 * If you cannot find this file you my obtain a copy of the GNU
 * General Public License by writing to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
 */
#
#define DO_NOT_ALLOW_ACCESS_TO_INTERNALS_OF_STRUCTS
#include <BuildDefines.h>
#ifdef DEBUG_BUILD
#define ERROR_CHECK_LEVEL ERROR_CHECK_FULL
#endif
#include <PalmOS.h>
#include "resources.h"
#include "bj.h"
#include "utils.h"
#include "constants.h"

static struct popup rules[] = {
	"Vegas",
	"Strip",
	"Reno",
	"A. City",
	"Custom",
	0
};

static struct popup decks[] = {
	"1",
	"2",
	"4",
	"8",
	0
};

static struct popup penetrate[] = {
	"80",
	"75",
	"60",
	0
};

static struct popup dblrules[] = {
	"Any",
	"9-11",
	"10,11",
	0
};

static struct popup surrender[] = {
	"No",
	"Early",
	"Late",
	0
};

static struct popup betstrat[] = {
	"None",
	"Martingale",
	"Bounce 1,2",
	"Cycle 1,2,3,5",
	0
};

void menustats()
{
	FormPtr form;
	char *cp;

	form = FrmInitForm(MenuStats);
	setfield(form, FieldHands, itoa(Hands, buf), 0);
	setfield(form, FieldHWon, itoa(Hwon, buf), 0);
	setfield(form, FieldHLost, itoa(Hlost, buf), 0);
	setfield(form, FieldMoney, itoa(Money, buf), 0);
	setfield(form, FieldMWon, itoa(Mwon, buf), 0);
	setfield(form, FieldMLost, itoa(Money - Mwon, buf), 0);
	buf[0] = '\0';
	if (Decisions) {
		itoa(((long)Dwon * 100)/Decisions, buf);
		cp = buf + StrLen(buf);
		*cp++ = '%';
		*cp++ = ' ';
		*cp++ = '(';
		itoa(Dwon, cp);
		cp = buf + StrLen(buf);
		*cp++ = '/';
		itoa(Decisions, cp);
		cp = buf + StrLen(buf);
		*cp++ = ')';
		*cp++ = 0;
	}
	setfield(form, FieldQuality, buf, 0);
	FrmDoDialog(form);
	FrmDeleteForm(form);
	return;
}

void menupref()
{
	FormPtr form;
	int odecks;

	form = FrmInitForm(MenuPref);
	setlbl(form, PopRules, rules[DefaultRules].name);
	odecks = DefDecks;
	setlbl(form, PopDecks, decks[DefDecks].name);
	setlbl(form, PopDepth, penetrate[DefDepth].name);
	setlbl(form, PopBetStrat, betstrat[BetStrat].name);
	setck(form, CheckDelay, DealDelay);
	setck(form, CheckNanny, Nanny);
	setfield(form, FieldPot, itoa(StartPot, buf), 0);
	setfield(form, FieldBet, itoa(BetMin, buf), 0);
	setfield(form, FieldMax, itoa(BetMax, buf), 0);
	if (FrmDoDialog(form) == QueryOK) {
		DealDelay = getck(form, CheckDelay);
		Nanny = getck(form, CheckNanny);
		StartPot = atoi(getfield(form, FieldPot, buf));
		BetMin = atoi(getfield(form, FieldBet, buf));
		BetMax = atoi(getfield(form, FieldMax, buf));
		BetStrat = getlbl(form, PopBetStrat, betstrat);
		DefDecks = getlbl(form, PopDecks, decks);
		Ndecks = atoi(decks[DefDecks].name);
		if (DefDecks != odecks)
			Decklim = 0;
		switch (DefDepth = getlbl(form, PopDepth, penetrate)) {

		case 0:		/* 80 */
			Depth = 80;
			break;

		case 1:		/* 75 */
			Depth = 75;
			break;

		case 2:		/* 60 */
			Depth = 60;
			break;

		}
		DefaultRules = getlbl(form, PopRules, rules);
		switch (DefaultRules) {

		case 0:		/* Vegas */
			DealerSoft17 = 1;
			DealerResplit = 1;
			DealerSplitace = 1;
			DealerDblSplit = 0;
			DblRule = 1;
			DealerDblAny = 1;
			DealerDblTen = 0;
			Surrender = 0;
			break;

		case 1:		/* Strip */
			DealerSoft17 = 0;
			DealerResplit = 1;
			DealerSplitace = 1;
			DealerDblSplit = 1;
			DblRule = 1;
			DealerDblAny = 1;
			DealerDblTen = 0;
			Surrender = 0;
			break;

		case 2:		/* Reno */
			DealerSoft17 = 1;
			DealerResplit = 1;
			DealerSplitace = 1;
			DealerDblSplit = 0;
			DblRule = 2;
			DealerDblAny = 0;
			DealerDblTen = 1;
			Surrender = 0;
			break;

		case 3:		/* Atlantic City */
			DealerSoft17 = 0;
			DealerResplit = 1;
			DealerSplitace = 1;
			DealerDblSplit = 1;
			DblRule = 0;
			DealerDblAny = 1;
			DealerDblTen = 0;
			Surrender = 0;
			break;

		} /* Don't do anything for "Custom" */
	}
	FrmDeleteForm(form);
	return;
}

void menurules()
{
	FormPtr form;

	form = FrmInitForm(MenuRules);
	setck(form, CheckSoft17, DealerSoft17);
	setck(form, CheckResplit, DealerResplit);
	setck(form, CheckSplitace, DealerSplitace);
	setck(form, CheckDblSplit, DealerDblSplit);
	setlbl(form, PopDouble, dblrules[DblRule].name);
	setlbl(form, PopSurrender, surrender[Surrender].name);
	if (FrmDoDialog(form) == QueryOK) {
		DealerSoft17 = getck(form, CheckSoft17);
		DealerResplit = getck(form, CheckResplit);
		DealerSplitace = getck(form, CheckSplitace);
		DealerDblSplit = getck(form, CheckDblSplit);
		switch(getlbl(form, PopDouble, dblrules)) {

		case 0:		/* Any */
			DealerDblAny = 1;
			DealerDblTen = 0;
			break;

		case 1:		/* 9 - 10 */
			DealerDblAny = 0;
			DealerDblTen = 0;
			break;

		case 2:		/* 11, 11 */
			DealerDblAny = 0;
			DealerDblTen = 1;
			break;

		}
		Surrender = getlbl(form, PopSurrender, surrender);
	}
	FrmDeleteForm(form);
	return;
}
