/*
 Copyright (c) 2000-2001, Neil Rhodes and Julie McKeehan
    neil@pobox.com
 All rights reserved.

From the book "Palm Programming, the Developer's Guide, 2nd edition"
by O'Reilly.

Permission granted to use this file however you see fit.
*/

/*
 Modified 2002 Don Dugger
*/

#define DO_NOT_ALLOW_ACCESS_TO_INTERNALS_OF_STRUCTS
#include <BuildDefines.h>
#ifdef DEBUG_BUILD
#define ERROR_CHECK_LEVEL ERROR_CHECK_FULL
#endif
#include <PalmOS.h>
#include "bj.h"
#include "utils.h"
#include "resources.h"

Err
RomVersionCompatible(UInt32 requiredVersion, UInt16 launchFlags)
{
	UInt32 romVersion;

	// See if we're on in minimum required version of the ROM or later.
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	if (romVersion < requiredVersion) {
		UInt16 safeToCallAlertFlags;

		safeToCallAlertFlags = sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp;
		if ((launchFlags & (safeToCallAlertFlags)) == safeToCallAlertFlags) {
			FrmAlert(RomIncompatibleAlert);

			// Pilot 1.0 will continuously relaunch this app unless we switch to 
			// another safe one.
			if (romVersion < sysMakeROMVersion(2, 0, 0, sysROMStageRelease, 0))
				AppLaunchWithCommand(sysFileCDefaultApp, sysAppLaunchCmdNormalLaunch, NULL);
		}

		return (sysErrRomIncompatible);
	}

	return errNone;
}

void *
GetObjectPtr(FormPtr form, UInt16 objectID)
{
	return FrmGetObjectPtr(form, FrmGetObjectIndex(form, objectID));
}

void *
GetObjectPtrFromActiveForm(UInt16 objectID)
{
	return GetObjectPtr(FrmGetActiveForm(), objectID);
}

char *getfield(FormPtr fp, int id, char *v)
{
	FieldPtr field;
	MemHandle h;

	*v = '\0';
	field = FrmGetObjectPtr(fp, FrmGetObjectIndex(fp, id));
	if ((h = FldGetTextHandle(field)) == NULL) {
		v[0] = '\0';
		return(v);
	}
	StrCopy(v, (char *) MemHandleLock(h));
	MemHandleUnlock(h);
	return(v);
}

void setfield(FormPtr fp, int id, char *v, int redraw)
{
	FieldPtr field;
	MemHandle h;

	field = FrmGetObjectPtr(fp, FrmGetObjectIndex(fp, id));
	if ((h = FldGetTextHandle(field))) {
		Err err;

		FldSetTextHandle(field, NULL);
		if ((err = MemHandleResize(h, StrLen(v) + 1))) {
			FldSetTextHandle(field, h);
			return;
		}
	} else if ((h = MemHandleNew(StrLen(v) + 1)) == NULL)
		return;
	StrCopy((char *) MemHandleLock(h), v);
	MemHandleUnlock(h);
	FldSetTextHandle(field, h);
	if (redraw)
		FldDrawField(field);
	return;
}

int getlbl(FormPtr fp, int id, struct popup *pp)
{
	int n;
	char *cp;

	id = FrmGetObjectIndex(fp, id);
	cp = (char *)CtlGetLabel(FrmGetObjectPtr(fp, id));
	n = 0;
	while (pp->name)
		if (StrCompare(cp, pp->name) == 0)
			return(n);
		else {
			n++;
			pp++;
		}
	return(n);
}

void setlbl(FormPtr fp, int id, char *v)
{

	id = FrmGetObjectIndex(fp, id);
	CtlSetLabel(FrmGetObjectPtr(fp, id), v);
	return;
}

int getck(FormPtr fp, int id)
{

	id = FrmGetObjectIndex(fp, id);
	return(FrmGetControlValue(fp, id));
}

int setck(FormPtr fp, int id, char v)
{

	id = FrmGetObjectIndex(fp, id);
	FrmSetControlValue(fp, id, v);
	return(v);
}

char *
itoa(int n, char *p)
{
	int i;
	char *ap;

	ap = p;
	if (n == 0) {
		*p++ = ' ';
		*p++ = '\0';
		return(ap);
	}
	if (n < 0) {
		*p++ = '-';
		n = -n;
	}
	i = n;
	while (i > 9) {
		i /= 10;
		p++;
	}
	*(p + 1) = '\0';
	do {
		*p-- = "0123456789"[n % 10];
		n /= 10;
	} while (n > 0);
	return (ap);
}

int atoi(char *cp)
{
	int n;
	char c;

	n = 0;
	while ((c = *cp++))
		if (c >= '0' && c <= '9')
			n = (n * 10) + (c - '0');
	return(n);
}

void delay(unsigned int ticks)
{
	unsigned int end, t;

	if (DealDelay) {
		end = TimGetTicks() + ticks;
		while ((t = TimGetTicks()) < end)
			;
	}
	return;
}
